package X10::Device;

use Data::Dumper;

use strict;

use X10::Log;

sub new
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   $self->{house_code} = $self->{controller}->{house_code} unless $self->{house_code};

   $self->{state} = 0;	# unknown state

   foreach (values %{$self->{macros}})
   {
      $_->controller($self->{controller});
   }

   return $self;
}


### manipulating the device:

sub on
{
   my $self = shift;
   return 0 unless $self->{controller};
   $self->{controller}->on($self);
}

sub off
{
   my $self = shift;
   return 0 unless $self->{controller};
   $self->{controller}->off($self);
}

sub dim
{
   my $self = shift;
   return 0 unless $self->{controller};
   $self->{controller}->dim($self);
}

sub bright
{
   my $self = shift;
   return 0 unless $self->{controller};
   $self->{controller}->bright($self);
}

sub got_event
{
   my $self = shift;
   my $func = uc(shift);

   if ($func eq 'ON')
   {
      if ($self->is_momentary)
      {
         x10_log('notice', "%s FIRING", $self->description);
      }
      elsif ($self->{state} == 1)
      {
         if (exists $self->{macros}->{on})
         {
            x10_log('debug', "Running ON macro");
            $self->{macros}->{on}->run;
         }
         elsif (!exists $self->{macros}->{on_transition})
         {
            x10_log('notice', "%s Still ON", $self->description);
         }

         $self->{state} = 1;
      }
      else
      {
         if (exists $self->{macros}->{on_transition})
         {
            $self->{macros}->{on_transition}->run;
         }
         elsif (exists $self->{macros}->{on})
         {
            $self->{macros}->{on}->run;
         }
         else
         {
            x10_log('notice', "%s Turning ON", $self->description);
         }

         $self->{state} = 1;
      }

   }
   elsif ($func eq 'OFF')
   {
      if ($self->{state} == -1)
      {
         if (exists $self->{macros}->{off})
         {
            $self->{macros}->{off}->run;
         }
         elsif (!exists $self->{macros}->{off_transition})
         {
            x10_log('notice', "%s Still OFF", $self->description);
         }
      }
      else
      {
         if (exists $self->{macros}->{off_transition})
         {
            $self->{macros}->{off_transition}->run;
         }
         elsif (exists $self->{macros}->{off})
         {
            $self->{macros}->{off}->run;
         }
         else
         {
            x10_log('notice', "%s Turning OFF", $self->description);
         }
      }

      $self->{state} = -1;
   }
   elsif ($func eq 'S1')
   {
      x10_log('notice', "%s Status ON", $self->description);
      $self->{state} = 1;
   }
   elsif ($func eq 'S0')
   {
      x10_log('notice', "%s Status OFF", $self->description);
      $self->{state} = -1;
   }
#   elsif ($func = 'DIM')
#   {
#   }
#   elsif ($func = 'BRIGHT')
#   {
#   }
}

### access methods

sub house_code
{
   my $self = shift;
   if (@_)
   {
      $self->{house_code} = uc(shift);
   }
   return $self->{house_code};
}

sub unit_code
{
   my $self = shift;
   if (@_)
   {
      $self->{unit_code} = shift;
   }
   return $self->{unit_code};
}

sub address
{
   my $self = shift;
   if (@_)
   {
      my $newaddr = shift;
      my ($hc, $uc) = $newaddr =~ /^([a-p])(\d{1,2})$/i;
      $self->house_code($1);
      $self->unit_code($2);
   }
   sprintf("%s%02s", $self->house_code, $self->unit_code);
}

sub controller
{
   my $self = shift;
   if (@_)
   {
      $self->{controller} = shift;

      foreach (values %{$self->{macros}})
      {
         $_->controller($self->{controller});
      }
   }
   $self->{controller};
}

sub description
{
   my $self = shift;
   $self->{description} || $self->address;
}

sub is_light
{
   my $self = shift;
   $self->{is_light};
}

sub is_momentary
{
   my $self = shift;
   $self->{is_momentary};
}

## abilities

sub can_dim
{
   my $self = shift;
   return (exists $self->{can_dim}) ? $self->{can_dim} : 0;
}

sub is_two_way
{
   my $self = shift;
   return (exists $self->{is_two_way}) ? $self->{is_two_way} : 0;
}

1;

