package X10::Log;

use Sys::Syslog;
use File::Basename;
use Exporter;

use strict;

use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(x10_log set_x10_log);

my $logtype = 'syslog';
my $minlevel = 3;

my %levels = (
	'debug'		=> 1,	# debug
	'info'		=> 2,	# informational
	'notice'	=> 3,	# normal, but significant
	'warning'	=> 4,	# warning
	'err'		=> 5,	# error
	'crit'		=> 6,	# critical
	'alert'		=> 7,	# action required immediately
	'emerg'		=> 8,	# system unusable
	);

BEGIN
{
   openlog('X10', 'pid', 'local5');
}

END
{
   closelog;
}

sub set_x10_log
{
   $logtype = shift;
   $minlevel = shift;

   $minlevel = $levels{$minlevel} || 2;
   $logtype = 'syslog' unless $logtype;
}

sub x10_log
{
   my $level = shift;
   my $format = shift;
   my @args = @_;

   $level = 'warning' unless exists $levels{$level};

   my $caller = caller || '';
   # $caller = '' if $caller eq 'main';

   if ($levels{$level} > 3)
   {
      $format = "($caller) " . $format;
   }

   if ($logtype eq 'syslog')
   {
      syslog($level, $format, @args);
   }
   # elsif ($logtype eq 'stderr')
   else
   {
      return unless ($levels{$level} >= $minlevel);

      printf STDERR $format . "\n", @args;
   }

}



1;

