package X10::Scheduler;

# this package implements a scheduler

use Data::Dumper;
use POSIX;

use strict;

use X10::Log;

sub new
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   return undef unless $self->{controller};

   x10_log('debug', "Constructing Scheduler");

   # this module doesn't do anything with events...
   # $self->{controller}->register_listener($self->event_callback);
   $self->{schedevents} = [];

   $self->read_config;

   return $self;
}

###

sub read_config
{
   my $self = shift;

   return unless $self->{configfile};

   $self->{config_lastmod} = 0 unless exists $self->{config_lastmod};

   x10_log('debug', "Checking %s config file...", __PACKAGE__);

   if ($self->{config_lastmod} == 0 || (stat($INC{$self->{configfile}}))[9] > $self->{config_lastmod})
   {
      $self->{schedevents} = [];
      $self->{config_lastmod} = time;

      x10_log('notice', "Reading scheduler config file...");

      delete $INC{$self->{configfile}};

      my $config = eval { require $self->{configfile} } || die $@;

      x10_log('notice', "Read %s scheduled events", scalar @$config);

      foreach (@$config)
      {
         $self->add( new X10::SchedEvent( %$_ ) );
      }

      x10_log('notice', "Loaded %s scheduled events successfully", scalar @{$self->{schedevents}});
   }
}

sub setup
{
   my $self = shift;

   if (@_)
   {
      while (my $key = shift)
      {
      }
   }

   return $self->{schedevents};
}

sub add
{
   my $self = shift;
   my $se = shift;

   return undef unless ($se && $se->isa('X10::SchedEvent'));

   $se->controller($self->{controller});
   $se->{latitude} = $self->{latitude} || 38.74274;
   $se->{longitude} = $self->{longitude} || -90.560143;

   x10_log('debug', "Queueing %s for %s",
	($se->description || 'unnamed event'),
	strftime("%a %b %e %H:%M %Y", localtime($se->next_time)),
	);

   push @{$self->{schedevents}}, $se;

   @{$self->{schedevents}} =
	sort { $a->next_time <=> $b->next_time }
	@{$self->{schedevents}};

   return 1;
}

sub next_event
{
   my $self = shift;
   return undef unless (@{$self->{schedevents}} > 0);
   return $self->{schedevents}->[0];
}

sub run
{
   my $self = shift;

   while ( @{$self->{schedevents}}
	&& $self->{schedevents}->[0]->next_time <= (int(time) + 30) )
   {
      my $se = shift @{$self->{schedevents}};

      $se->run;

      if ($se->reschedule)
      {
         $self->add($se);
      }
      else
      {
      }
   }
}

# nothing to do with events
sub event_callback
{
   my $self = shift;
   return sub { };
}

# no fds to deal with
sub select_fds
{
   return ();
}


1;

