package X10::Server;

# this is a network server object that accepts connections via a TCP
# socket and relays the 'event requests' to an
# X10::Controller-type object

use File::Basename;
use FileHandle;
use IO::Socket;

use strict;

use X10::Event;
use X10::EventList;
use X10::Log;
use X10::ClientConnection;

sub new
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   return undef unless ( $self->{controller} );

   $self->{server_port} ||= 2020;

   x10_log('debug', "Using TCP port %s", $self->{server_port});

   $self->{listen_socket} = new IO::Socket(
        Domain => &AF_INET,
        Proto => 'tcp',
        LocalPort => $self->{server_port},
        Listen => 5,
        Reuse => 1,
        MultiHomed => 1,
        );

   unless ($self->{listen_socket})
   {
      warn "Problem listening on socket: ", $!;
      return undef;
   }

   $self->{clients} = {};

   # $self->{controller}->register_listener($self->event_callback);

   $SIG{PIPE} = sub {};		# Ignore SIGPIPE

   return $self;
}

sub select_fds
{
   my $self = shift;

   foreach my $key (keys %{$self->{clients}})
   {
      delete $self->{clients}->{$key}
	unless $self->{clients}->{$key}->is_open;
   }

   return (keys %{$self->{clients}}, $self->{listen_socket}->fileno);
}

sub handle_input
{
   my $self = shift;
   my $fd = shift;

   if ($fd == $self->{listen_socket}->fileno)
   {
      my $newsocket = $self->{listen_socket}->accept;

      x10_log('debug', "New connection on %s", $newsocket->fileno);

      $self->{clients}->{$newsocket->fileno} = new X10::ClientConnection($newsocket, $self->{controller});
   }
   elsif (exists $self->{clients}->{$fd})
   {
      $self->{clients}->{$fd}->handle_input;
   }
   else
   {
      x10_log('err', "Don't know anything about FD %s", $fd);
   }

}


###



1;

